@@ -22,41 +22,56 @@ import org.apache.commons.pool2.ObjectPool;
 
 import java.sql.Connection;
 import java.sql.SQLException;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
 
 /**
- * ManagedConnection is responsible for managing a database connection in a transactional environment
- * (typically called "Container Managed").  A managed connection operates like any other connection
- * when no global transaction (a.k.a. XA transaction or JTA Transaction) is in progress.  When a
- * global transaction is active a single physical connection to the database is used by all
- * ManagedConnections accessed in the scope of the transaction.  Connection sharing means that all
- * data access during a transaction has a consistent view of the database.  When the global transaction
- * is committed or rolled back the enlisted connections are committed or rolled back.  Typically upon
- * transaction completion, a connection returns to the auto commit setting in effect before being
- * enlisted in the transaction, but some vendors do not properly implement this.
+ * ManagedConnection is responsible for managing a database connection in a transactional environment (typically called
+ * "Container Managed"). A managed connection operates like any other connection when no global transaction (a.k.a. XA
+ * transaction or JTA Transaction) is in progress. When a global transaction is active a single physical connection to
+ * the database is used by all ManagedConnections accessed in the scope of the transaction. Connection sharing means
+ * that all data access during a transaction has a consistent view of the database. When the global transaction is
+ * committed or rolled back the enlisted connections are committed or rolled back. Typically upon transaction
+ * completion, a connection returns to the auto commit setting in effect before being enlisted in the transaction, but
+ * some vendors do not properly implement this.
+ * <p>
+ * When enlisted in a transaction the setAutoCommit(), commit(), rollback(), and setReadOnly() methods throw a
+ * SQLException. This is necessary to assure that the transaction completes as a single unit.
+ * </p>
  *
- * When enlisted in a transaction the setAutoCommit(), commit(), rollback(), and setReadOnly() methods
- * throw a SQLException.  This is necessary to assure that the transaction completes as a single unit.
+ * @param <C>
+ *            the Connection type
  *
- * @param <C> the Connection type
- *
- * @author Dain Sundstrom
- * @version $Id: ManagedConnection.java 1658616 2015-02-10 02:49:54Z psteitz $
  * @since 2.0
  */
 public class ManagedConnection<C extends Connection> extends DelegatingConnection<C> {
+
     private final ObjectPool<C> pool;
     private final TransactionRegistry transactionRegistry;
     private final boolean accessToUnderlyingConnectionAllowed;
     private TransactionContext transactionContext;
     private boolean isSharedConnection;
+    private final Lock lock;
 
-    public ManagedConnection(ObjectPool<C> pool,
-            TransactionRegistry transactionRegistry,
-            boolean accessToUnderlyingConnectionAllowed) throws SQLException {
+    /**
+     * Constructs a new instance responsible for managing a database connection in a transactional environment.
+     *
+     * @param pool
+     *            The connection pool.
+     * @param transactionRegistry
+     *            The transaction registry.
+     * @param accessToUnderlyingConnectionAllowed
+     *            Whether or not to allow access to the underlying Connection.
+     * @throws SQLException
+     *             Thrown when there is problem managing transactions.
+     */
+    public ManagedConnection(final ObjectPool<C> pool, final TransactionRegistry transactionRegistry,
+            final boolean accessToUnderlyingConnectionAllowed) throws SQLException {
         super(null);
         this.pool = pool;
         this.transactionRegistry = transactionRegistry;
         this.accessToUnderlyingConnectionAllowed = accessToUnderlyingConnectionAllowed;
+        this.lock = new ReentrantLock();
         updateTransactionStatus();
     }
 
@@ -68,7 +83,7 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
 
     private void updateTransactionStatus() throws SQLException {
         // if there is a is an active transaction context, assure the transaction context hasn't changed
-        if (transactionContext != null) {
+        if (transactionContext != null && !transactionContext.isTransactionComplete()) {
             if (transactionContext.isActive()) {
                 if (transactionContext != transactionRegistry.getActiveTransactionContext()) {
                     throw new SQLException("Connection can not be used while enlisted in another transaction");
@@ -77,7 +92,7 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
             }
             // transaction should have been cleared up by TransactionContextListener, but in
             // rare cases another lister could have registered which uses the connection before
-            // our listener is called.  In that rare case, trigger the transaction complete call now
+            // our listener is called. In that rare case, trigger the transaction complete call now
             transactionComplete();
         }
 
@@ -90,16 +105,16 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
             // in the transaction, replace our delegate with the enrolled connection
 
             // return current connection to the pool
-            C connection = getDelegateInternal();
+            final C connection = getDelegateInternal();
             setDelegate(null);
             if (connection != null) {
                 try {
                     pool.returnObject(connection);
-                } catch (Exception ignored) {
+                } catch (final Exception ignored) {
                     // whatever... try to invalidate the connection
                     try {
                         pool.invalidateObject(connection);
-                    } catch (Exception ignore) {
+                    } catch (final Exception ignore) {
                         // no big deal
                     }
                 }
@@ -112,7 +127,7 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
             // always be of type C since it has been shared by another
             // connection from the same pool.
             @SuppressWarnings("unchecked")
-            C shared = (C) transactionContext.getSharedConnection();
+            final C shared = (C) transactionContext.getSharedConnection();
             setDelegate(shared);
 
             // remember that we are using a shared connection so it can be cleared after the
@@ -126,7 +141,7 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
                     // borrow a new connection from the pool
                     connection = pool.borrowObject();
                     setDelegate(connection);
-                } catch (Exception e) {
+                } catch (final Exception e) {
                     throw new SQLException("Unable to acquire a new connection from the pool", e);
                 }
             }
@@ -139,12 +154,12 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
                 // register our connection as the shared connection
                 try {
                     transactionContext.setSharedConnection(connection);
-                } catch (SQLException e) {
+                } catch (final SQLException e) {
                     // transaction is hosed
                     transactionContext = null;
                     try {
                         pool.invalidateObject(connection);
-                    } catch (Exception e1) {
+                    } catch (final Exception e1) {
                         // we are try but no luck
                     }
                     throw e;
@@ -162,23 +177,30 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
             try {
                 // Don't actually close the connection if in a transaction. The
                 // connection will be closed by the transactionComplete method.
-                if (transactionContext == null) {
+                //
+                // DBCP-484 we need to make sure setClosedInternal(true) being
+                // invoked if transactionContext is not null as this value will
+                // be modified by the transactionComplete method which could run
+                // in the different thread with the transaction calling back.
+                lock.lock();
+                if (transactionContext == null || transactionContext.isTransactionComplete()) {
                     super.close();
                 }
             } finally {
                 setClosedInternal(true);
+                lock.unlock();
             }
         }
     }
 
     /**
-     * Delegates to {@link ManagedConnection#transactionComplete()}
-     * for transaction completion events.
+     * Delegates to {@link ManagedConnection#transactionComplete()} for transaction completion events.
+     *
      * @since 2.0
      */
     protected class CompletionListener implements TransactionContextListener {
         @Override
-        public void afterCompletion(TransactionContext completedContext, boolean commited) {
+        public void afterCompletion(final TransactionContext completedContext, final boolean commited) {
             if (completedContext == transactionContext) {
                 transactionComplete();
             }
@@ -186,7 +208,9 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
     }
 
     protected void transactionComplete() {
-        transactionContext = null;
+        lock.lock();
+        transactionContext.completeTransaction();
+        lock.unlock();
 
         // If we were using a shared connection, clear the reference now that
         // the transaction has completed
@@ -197,7 +221,7 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
 
         // If this connection was closed during the transaction and there is
         // still a delegate present close it
-        Connection delegate = getDelegateInternal();
+        final Connection delegate = getDelegateInternal();
         if (isClosedInternal() && delegate != null) {
             try {
                 setDelegate(null);
@@ -205,7 +229,7 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
                 if (!delegate.isClosed()) {
                     delegate.close();
                 }
-            } catch (SQLException ignored) {
+            } catch (final SQLException ignored) {
                 // Not a whole lot we can do here as connection is closed
                 // and this is a transaction callback so there is no
                 // way to report the error.
@@ -218,14 +242,13 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
     //
 
     @Override
-    public void setAutoCommit(boolean autoCommit) throws SQLException {
+    public void setAutoCommit(final boolean autoCommit) throws SQLException {
         if (transactionContext != null) {
             throw new SQLException("Auto-commit can not be set while enrolled in a transaction");
         }
         super.setAutoCommit(autoCommit);
     }
 
-
     @Override
     public void commit() throws SQLException {
         if (transactionContext != null) {
@@ -242,9 +265,8 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
         super.rollback();
     }
 
-
     @Override
-    public void setReadOnly(boolean readOnly) throws SQLException {
+    public void setReadOnly(final boolean readOnly) throws SQLException {
         if (transactionContext != null) {
             throw new SQLException("Read-only can not be set while enrolled in a transaction");
         }
@@ -257,6 +279,7 @@ public class ManagedConnection<C extends Connection> extends DelegatingConnectio
 
     /**
      * If false, getDelegate() and getInnermostDelegate() will return null.
+     *
      * @return if false, getDelegate() and getInnermostDelegate() will return null
      */
     public boolean isAccessToUnderlyingConnectionAllowed() {
