@@ -28,10 +28,10 @@ public class TesterUtils {
      * Access a private field. Do it this way rather than increasing the
      * visibility of the field in the public API.
      */
-    public static Object getField(Object target, String fieldName)
+    public static Object getField(final Object target, final String fieldName)
             throws Exception {
-        Class<?> clazz = target.getClass();
-        Field f = clazz.getDeclaredField(fieldName);
+        final Class<?> clazz = target.getClass();
+        final Field f = clazz.getDeclaredField(fieldName);
         f.setAccessible(true);
         return f.get(target);
     }
